@@ -31,6 +31,7 @@
 //!  * Validating directory contents
 //!
 
+use regex::Regex;
 use std::ffi::OsStr;
 use std::io;
 use std::path::{Path, PathBuf};
@@ -39,8 +40,15 @@ use tempfile::TempDir;
 
 use bintest::BinTest;
 
-// A TestCall object binds a BinTest::Command to a single executable and environment and
-// provides functions to call this multiple times.
+/// exporting things for convinience
+pub mod prelude {
+    pub use crate::TestCall;
+    pub use crate::TestDir;
+    pub use crate::TestOutput;
+}
+
+/// A TestCall object binds a BinTest::Command to a single executable and environment and
+/// provides functions to call this multiple times.
 pub struct TestCall<'a> {
     executables: &'a BinTest,
     name: &'static str,
@@ -49,7 +57,7 @@ pub struct TestCall<'a> {
 }
 
 impl<'a> TestCall<'a> {
-    // Creates a new testcall object for the executable 'name'
+    /// Creates a new testcall object for the executable 'name'
     pub fn new(executables: &'a BinTest, name: &'static str) -> TestCall<'a> {
         TestCall {
             executables,
@@ -58,14 +66,14 @@ impl<'a> TestCall<'a> {
         }
     }
 
-    // Sets the current dir in which the next call shall execute
+    /// Sets the current dir in which the next call shall execute
     pub fn current_dir(&mut self, dir: Box<dyn TestDir>) -> &mut Self {
         self.dir = Some(dir);
         self
     }
 
-    // Calls the executable with the given arguments and expects successful exit.
-    // Returns a TestOutput object for further investigation.
+    /// Calls the executable with the given arguments and expects successful exit.
+    /// Returns a TestOutput object for further investigation.
     #[track_caller]
     pub fn call<I, S>(&self, args: I) -> Output
     where
@@ -84,23 +92,64 @@ impl<'a> TestCall<'a> {
 
 /// Augment std::process::Output with testing and assertions
 pub trait TestOutput {
+    /// Will panic when the program did not exited successful.
     #[track_caller]
     fn assert_success(&self) -> &Self;
 
+    /// Expects that the program exited with a failure.
     #[track_caller]
     fn assert_failure(&self) -> &Self;
+
+    /// Expects that the program exited with the provided code.
+    #[track_caller]
+    fn assert_exitcode(&self, code: i32) -> &Self;
+
+    /// Applies a regex match check to stdout, will panic when the match failed.
+    #[track_caller]
+    fn assert_stdout(&self, regex: &str) -> &Self;
+
+    /// Applies a regex match check to stderr, will panic when the match failed.
+    #[track_caller]
+    fn assert_stderr(&self, regex: &str) -> &Self;
 }
 
 impl TestOutput for Output {
-    #[track_caller]
     fn assert_success(&self) -> &Self {
-        assert!(self.status.success(), "expected successful exit status");
+        assert!(self.status.success(), "expected success at exit");
         self
     }
 
-    #[track_caller]
     fn assert_failure(&self) -> &Self {
-        assert_eq!(self.status.success(), false, "expected failure at exit");
+        assert!(!self.status.success(), "expected failure at exit");
+        self
+    }
+
+    fn assert_exitcode(&self, code: i32) -> &Self {
+        assert_eq!(self.status.code(), Some(code), "unexpected exitcode");
+        self
+    }
+
+    fn assert_stdout(&self, regex: &str) -> &Self {
+        let re = Regex::new(regex).expect("compiled regex");
+        let text = String::from_utf8_lossy(&self.stdout);
+        assert!(
+            re.is_match(&text),
+            "stdout does not match:\n{}\nstdout was:\n{}",
+            regex,
+            text
+        );
+        self
+    }
+
+    fn assert_stderr(&self, regex: &str) -> &Self {
+        let re = Regex::new(regex).expect("compiled regex");
+        let text = String::from_utf8_lossy(&self.stderr);
+        assert!(
+            re.is_match(&text),
+            "stderr does not match:\n{}\nstdout was:\n{}",
+            regex,
+            text
+        );
         self
     }
 }
